#!/bin/bash

#$1 - file-name

set -e

if [ $# -ne 1 ]; then
  echo "Usage: $0 <file-name>"
  exit 1
fi

renice 19 -p $$ &> /dev/null

FILE_NAME=$1.tar.gz
DIR=saved_files

#clean up before we start
cd /tmp;
/bin/rm -rf $DIR *.tar.gz 2> /dev/null
cd - > /dev/null

mkdir -p /tmp/$DIR/cores
mkdir -p /tmp/$DIR/other/additional_logs
mkdir -p /tmp/$DIR/tcrcli
mkdir -p /tmp/$DIR/telegraf

function handle_ctrl_c {
  echo "\nCaught Ctrl+C, cleaning up and exiting..."
  cleanup_and_exit 1
}

trap handle_ctrl_c SIGINT

function cleanup_and_exit {
  exit_code=$1
  if [ -z $1 ]
  then
    exit_code=0
  fi

  /bin/rm -rf $DIR 2> /dev/null
  exit $1
}

function create_tar {
  available_space_mb=`df / -m | grep /$ | awk '{print $4}'`
  files_disk_space=`du -sm $DIR | awk '{print $1}'`
  required_space=$((files_disk_space * 2))
  minimal_disk_space=100 # 100MB

  if [ $required_space -gt $((available_space_mb - minimal_disk_space)) ]
  then
    echo "Not enough disk space to compress TCR logs"
    cleanup_and_exit 1
  fi

  /bin/tar czf $FILE_NAME $DIR/ 2> /dev/null
}

function copy_file {
  # Iterate all args and extract all args except the last as source, and the last as destination
  unset src_files
  while [ $# -gt 1 ]
  do
    src_files+=("$1")
    shift
  done

  src=${src_files[*]}
  dst=$1

  available_space_mb=`df -m / | grep /$ | awk '{print $4}'`
  required_space=`du -sm $src 2>/dev/null | awk '{sum+=$1} END {printf "%d", sum}'`
  minimal_disk_space=100 # 100MB

  if [[ $required_space -gt $((available_space_mb - minimal_disk_space)) ]]
  then
    echo "Not enough disk space to collect TCR logs"
    cleanup_and_exit 1
  fi

  /bin/cp -r $src $dst 2>/dev/null || :;
}

function copy_files {
  TCR_DIR=`basename $1`;
  mkdir -p /tmp/$DIR/$TCR_DIR
  
  #copy configuration files
  copy_file /opt/$TCR_DIR/{tcr.conf,tcr_conf_active.json,tcr_conf_passive.json,tdns_persistent.txt} /tmp/$DIR/$TCR_DIR
  copy_file /opt/$TCR_DIR/wanproxy/wanproxy.conf /tmp/$DIR/$TCR_DIR

  #copy iptables and ipset files
  copy_file /opt/$TCR_DIR/{ipset_set,iptables_set,tcr_rules} /tmp/$DIR/$TCR_DIR
  
  #copy all logs 
  copy_file /var/log/$TCR_DIR/*.log* /tmp/$DIR/$TCR_DIR
  
  #copy wanproxy core dumps files
  copy_file /opt/$TCR_DIR/wanproxy/var/log/* /tmp/$DIR/$TCR_DIR

  #delete empty folders
  find /tmp/$DIR/$TCR_DIR/ -type d -empty -delete 2>/dev/null || :;
}

#loop through all TCR folders in /opt/
for f in /opt/*;
do
  if [[ -d "$f" && ! -L "$f" ]];
  then
    if [[ ${f} == /opt/tcr-* ]];
    then
	    copy_files $f;
    fi	
  fi
done

#copy core dump files
copy_file /var/log/core.* /tmp/$DIR/cores
copy_file /var/crash/* /tmp/$DIR/cores
copy_file /var/log/tcr_cleaner.log* /tmp/$DIR/other

#gre persistency file
copy_file /etc/tcr_gre_sites_persist.json /tmp/$DIR/other

#additional outputs folder
copy_file /var/log/additional_logs/* /tmp/$DIR/other/additional_logs

#copy system logs
copy_file /var/log/{dmesg*,messages*,secure*,yum.log*,pluto*,bird*,gred*} /tmp/$DIR/other

#copy SELinux alerts logs
copy_file /var/log/audit/audit.log* /tmp/$DIR/other

#mask customer ips
mask=`tcr_cli show General.mask_client_address | awk '{print $3;}' | sed '$s/.$//'`
if [[ "${mask}" == "true" && -f /tmp/$DIR/other/pluto* ]];
then
  ls -d /tmp/$DIR/other/pluto* | xargs sed -i -e '/\@Customer/s/\(\([0-9]\{1,3\}\.\)\{3\}\)[0-9]\{1,3\}/\1XXX/g' || :;
fi

#copy ops logs
copy_file /var/log/tcrcli/tcr_cli.log* /tmp/$DIR/tcrcli
copy_file /var/log/telegraf/telegraf.log* /tmp/$DIR/telegraf

#create tar.gz file
cd /tmp;
create_tar

#clean up
cleanup_and_exit